import os
import cv2
import csv
import math
import numpy as np
import tifffile
import matplotlib.pyplot as plt
from sklearn.cluster import KMeans
from PyQt5.QtWidgets import QMessageBox
from matplotlib.offsetbox import OffsetImage, AnnotationBbox

import warnings
warnings.filterwarnings('ignore')


def bw_correction(image, rectangle, color):
    if np.max(image) <= 1:
        image = np.asarray(image*255).astype('uint8')
    if color == 'black':
        image = 255 - image
        
    if rectangle != None:
        rectangle = rectangle.data
        x0 = int(rectangle[0][0][0])
        y0 = int(rectangle[0][0][1])
        x1 = int(rectangle[0][2][0])
        y1 = int(rectangle[0][2][1])

        image_patch = image[x0:x1, y0:y1]
    else:
        image_patch = image
    image_max = (image*1.0 / image_patch.max(axis=(0, 1))).clip(0, 1)
    image_max = np.asarray(image_max*255).astype('uint8')
    
    if color == 'black':
        image_max = 255 - image_max

    return image_max


def rgb_correction(image, rectangle, color):
    if color == 'red':
        index = 0
    elif color == 'green':
        index = 1
    elif color == 'blue':
        index = 2
    if np.max(image) <= 1:
        image = np.asarray(image*255).astype('uint8')
    if rectangle != None:
        rectangle = rectangle.data
        x0 = int(rectangle[0][0][0])
        y0 = int(rectangle[0][0][1])
        x1 = int(rectangle[0][2][0])
        y1 = int(rectangle[0][2][1])

        image_patch = image[x0:x1, y0:y1, index]
    else:
        image_patch = image[:,:,index]
    image_max = (image[:,:,index]*1.0 / image_patch.max(axis=(0, 1))).clip(0, 1)

    im_res = np.zeros(image.shape, dtype=np.uint8)
    for i in range(3):
        if i == index:
            im_res[:,:,i] = np.asarray(image_max*255).astype('uint8')
        else:
            im_res[:,:,i] = image[:,:,i]
    return im_res


def cmy_correction(image, rectangle, color):
    if color == 'yellow':
        indexes = [0, 1]
    elif color == 'magenta':
        indexes = [0, 2]
    elif color == 'cyan':
        indexes = [1, 2]
    if np.max(image) <= 1:
        image = np.asarray(image*255).astype('uint8')
    
    if rectangle != None:
        rectangle = rectangle.data
        x0 = int(rectangle[0][0][0])
        y0 = int(rectangle[0][0][1])
        x1 = int(rectangle[0][2][0])
        y1 = int(rectangle[0][2][1])

    im_res = np.zeros(image.shape, dtype=np.uint8)
    for index in indexes:
        image_copy = image.copy()
        if rectangle != None:
            image_patch = image[x0:x1, y0:y1, index]
        else:
            image_patch = image[:,:,index]
        image_max = (image_copy[:,:,index]*1.0 / image_patch.max(axis=(0, 1))).clip(0, 1)
        
        for i in range(3):
            if i == index:
                im_res[:,:,i] = np.asarray(image_max*255).astype('uint8')
    
    for i in range(3):
        if i not in indexes:
            im_res[:,:,i] = image[:,:,i]
        
    return im_res

################################################################

def set_cm_scale(line):
    d = math.dist(line[0][0],line[0][1])
    # Message pop-up when analysis is finished
    msg = QMessageBox() 
    msg.setIcon(QMessageBox.Information)
    msg.setText("Scale saved!")
    msg.setStandardButtons(QMessageBox.Ok | QMessageBox.Cancel)
    msg.exec_() 
    return d

################################################################

def color_analysis(image, mask_init, output_path, image_name, scale):
    support_dir = os.path.join(output_path, 'support_images')
    if not os.path.exists(support_dir):
        os.mkdir(support_dir)

    # Read the image and the mask correctly and find contour
    if np.max(image) <= 1:
        image = np.asarray(image*255).astype('uint8')
    mask_init = mask_init.astype('uint8')

    # Keep only the bigger contour found
    cnts, _ = cv2.findContours(mask_init, cv2.RETR_EXTERNAL, cv2.CHAIN_APPROX_NONE)
    cnt = max(cnts, key=cv2.contourArea)
    out = np.zeros(mask_init.shape, np.uint8)
    mask = cv2.drawContours(out, [cnt], -1, 255, cv2.FILLED)
    masktosave = cv2.drawContours(out, [cnt], -1, 255, cv2.FILLED)
    # Save support-composite image
    composite = np.zeros((2, image.shape[0], image.shape[1], image.shape[2]), dtype=np.uint8)
    composite[0] = image
    composite[1,:,:,0] = masktosave
    composite[1,:,:,1] = masktosave
    composite[1,:,:,2] = masktosave
    tifffile.imwrite(os.path.join(support_dir, image_name+'.tif'), composite, imagej=True)

    contours, _ = cv2.findContours(mask,cv2.RETR_EXTERNAL,cv2.CHAIN_APPROX_SIMPLE)
    cnt = contours[0]
    x,y,w,h = cv2.boundingRect(cnt)

    # Mask the image with the mask
    masked_image = cv2.bitwise_and(image, image, mask=mask)
    contours, _ = cv2.findContours(mask,cv2.RETR_EXTERNAL,cv2.CHAIN_APPROX_SIMPLE)
    cnt = contours[0]
    x,y,w,h = cv2.boundingRect(cnt)

    # Find lenght of the fish if the scale was set
    fish_len_px, masked_image, rotated_mask = measure_fish(cnt, masked_image, mask)
    if scale:
        fish_len_cm = fish_len_px/scale # scale = pixels per cm
    else:
        fish_len_cm = None

    # Crop the image around the fish mask
    masked_cropped = masked_image[y:y+h,x:x+w]
    
    # Color analysis which autom^nomsly save figures
    n_black_outofmask = (mask[y:y+h,x:x+w].shape[0]*mask[y:y+h,x:x+w].shape[1]) - np.count_nonzero(mask[y:y+h,x:x+w])
    try: colors_plot(masked_cropped, output_path, image_name, n_black_outofmask)
    except: pass
    try: colors_distribution_pie(masked_cropped, rotated_mask[y:y+h,x:x+w], output_path, image_name, n_black_outofmask, fish_len_px, fish_len_cm)
    except: pass

    # Message pop-up when analysis is finished
    msg = QMessageBox() 
    msg.setIcon(QMessageBox.Information)
    msg.setText("Analysis saved!")
    msg.setStandardButtons(QMessageBox.Ok | QMessageBox.Cancel)
    msg.exec_() 


def measure_fish(contour, masked_image, mask):
    hh, ww = masked_image.shape[:2]
    rotrect = cv2.minAreaRect(contour)
    (center), (width,height), angle = rotrect
    # Draw over the imagr the rectangle
    #box = cv2.boxPoints(rotrect)
    #boxpts = np.int0(box)
    #cv2.drawContours(masked_image,[boxpts],0,(255,0,255),10)
    if angle < -45:
        angle = -angle
    # otherwise, check width vs height
    else:
        if width > height:
            angle = -angle
        
        else:
            angle = -(90 + angle)
            
    # Get rotation matrix
    M = cv2.getRotationMatrix2D(center, -angle, scale=1.0)

    # Rotate image
    #masked_image = cv2.warpAffine(masked_image, M, (ww, hh), flags=cv2.INTER_CUBIC, borderMode=cv2.BORDER_REPLICATE)
    #mask = cv2.warpAffine(mask, M, (ww, hh), flags=cv2.INTER_CUBIC, borderMode=cv2.BORDER_REPLICATE)

    return np.max([width,height]), masked_image, mask


def colors_plot(masked_cropped, output_path, image_name, n_black_outofmask):
    def find_minimum_color(colors):
        # Function to calculate the sum of RGB components
        def rgb_sum(color):
            return sum(color)

        # Use the min function with a key parameter to find the color with the smallest sum of components
        min_color = min(colors, key=rgb_sum)
        return min_color
    
    # Preprocess the image
    masked_cropped = masked_cropped.reshape((masked_cropped.shape[0] * masked_cropped.shape[1], 3))
    # Apply K-means clustering
    num_colors = 5  # Number of colors to extract
    kmeans = KMeans(n_clusters=num_colors)
    kmeans.fit(masked_cropped)

    # Get the colors
    colors = kmeans.cluster_centers_
    labels = kmeans.labels_
    
    # Create a histogram of the color labels
    label_counts = np.bincount(labels)
    
    # Sort colors by frequency
    sorted_indices = np.argsort(label_counts)[::-1]
    colors = colors[sorted_indices]
    label_counts = label_counts[sorted_indices]
    
    # Create a figure
    plt.figure(figsize=(8, 6))
    
    # Plot each color as a bar
    black = find_minimum_color(colors)
    tosave_colors = []
    for i, color in enumerate(colors):
        tosave_colors.append([int(c) for c in color])
        color = [abs(c) for c in color]
        if np.sum(color) == np.sum(black):
            plt.barh(i, label_counts[i]-n_black_outofmask, color=np.array(color)/255, edgecolor='black')
        else:
            plt.barh(i, label_counts[i], color=np.array(color)/255, edgecolor='black')
    
    
    plt.xlabel('Pixel Count')
    plt.ylabel('Color')
    plt.title('Colors in Image')
    plt.yticks(np.arange(0, num_colors), tosave_colors)
    plt.savefig(os.path.join(output_path, image_name+'_colors_plot'), bbox_inches='tight', dpi=200)
    plt.close()


def colors_distribution_pie(masked_image, cropped_mask, output_path, image_name, n_black_outofmask, fish_len_px, fish_len_cm):
    def classify_color(pixel):
        # Define thresholds for black, white, and orange in RGB
        black_threshold = 100
        white_threshold = 200
        white_lower = np.array([0, 100, 200]) # which is almost blue becuase most of the time the background is light blue
        orange_lower = np.array([100, 40, 0])
        orange_upper = np.array([255, 230, 65])
        
        if np.all(pixel <= black_threshold):
            return 'black'
        elif np.all(pixel >= white_threshold):
            return 'white'
        elif np.all(pixel >= orange_lower) and np.all(pixel <= orange_upper):
            return 'orange'
        else:
            if np.all(pixel >= white_lower):
                return 'white'
            return 'black'

    # Classify each pixel in the image
    black_count = 0
    white_count = 0
    orange_count = 0

    masked_image_copy = masked_image.copy()
    res_image = masked_image.copy()

    for x in range(masked_image_copy.shape[1]):
        for y in range(masked_image_copy.shape[0]):
            pixel = masked_image_copy[y,x,:]
            color_class = classify_color(pixel)
            if color_class == 'black':
                res_image[y,x,:] = (0,0,0)
                black_count += 1
            elif color_class == 'white':
                res_image[y,x,:] = (255,255,255)
                white_count += 1
            elif color_class == 'orange':
                res_image[y,x,:] = (255,140,0)
                orange_count += 1
            elif color_class == 'other':
                res_image[y,x,:] = (0,140,255)
                orange_count += 1
    
    n_black = black_count-n_black_outofmask
    tot_px = n_black + white_count + orange_count
    if ('colors.csv' in os.listdir(output_path)):
        with open(os.path.join(output_path, 'colors.csv'), "a") as file:
            writer = csv.writer(file)
            writer.writerow([image_name, n_black, white_count, orange_count, round((n_black)/tot_px, 2)*100, round((white_count)/tot_px, 2)*100, round((orange_count)/tot_px, 2)*100, fish_len_px, fish_len_cm])
    else:
        with open(os.path.join(output_path, 'colors.csv'), "w") as file:
            writer = csv.writer(file)
            writer.writerow(['Image', '#black [px]', '#white [px]', '#orange [px]', 'black[%]', 'white[%]', 'orange[%]', 'len fish [px]', 'len fish [cm]'])
            writer.writerow([image_name, n_black, white_count, orange_count, round((n_black)/tot_px, 2)*100, round((white_count)/tot_px, 2)*100, round((orange_count)/tot_px, 2)*100, fish_len_px, fish_len_cm])


    # Step 4: Plot the results as a donut chart
    labels = ['Black', 'White', 'Orange']
    sizes = [black_count-n_black_outofmask, white_count, orange_count]
    colors = ['#000000', '#FFFFFF', '#FF8C00']

    fig, ax = plt.subplots()
    ax.pie(sizes, labels=labels, autopct='%1.1f%%', colors=colors, startangle=90, wedgeprops=dict(width=0.3, edgecolor='gray'))
    # Draw a circle at the center to make it look like a donut
    center_circle = plt.Circle((0, 0), 0.70, fc='white')
    fig.gca().add_artist(center_circle)

    contours, _ = cv2.findContours(cropped_mask, cv2.RETR_TREE, cv2.CHAIN_APPROX_SIMPLE)
    cv2.drawContours(res_image, contours, -1, (50,50,50), 2)

    # Add the image to the center of the donut
    res_image[cropped_mask == 0] = (255,255,255)
    imagebox = OffsetImage(res_image, zoom=0.05)
    ab = AnnotationBbox(imagebox, (0, 0), frameon=False)
    ax.add_artist(ab)

    ax.axis('equal') # Equal aspect ratio ensures that pie is drawn as a circle
    plt.title('Dominant Colors Distribution')
    plt.savefig(os.path.join(output_path, image_name+'_colors_distribution'), bbox_inches='tight', dpi=200)
    plt.close()

################################################################

def lock_controls(layer, viewer, widget_list, locked=True):
    qctrl = viewer.window.qt_viewer.controls.widgets[layer]
    for wdg in widget_list:
        getattr(qctrl, wdg).setEnabled(not locked)
        # or setVisible() if you want to just hide them completely

################################################################

"""#image = tifffile.imread('/Users/aravera/Documents/PROJECTS/DSB_Salamin/Lucy_s35/prova.tif')
image = np.asarray(Image.open('/Users/aravera/Documents/PROJECTS/DSB_Salamin/Lucy_s35/res/Corrected_Apo1_4_colors.png'))
mask = np.asarray(Image.open('/Users/aravera/Documents/PROJECTS/DSB_Salamin/Lucy_s35/res/mask_Apo1_4.png'))

masked_image = cv2.bitwise_and(image, image, mask=mask)
contours, _ = cv2.findContours(mask,cv2.RETR_EXTERNAL,cv2.CHAIN_APPROX_SIMPLE)
cnt = contours[0]
x,y,w,h = cv2.boundingRect(cnt)
n_black_outofmask = (mask[y:y+h,x:x+w].shape[0]*mask[y:y+h,x:x+w].shape[1]) - np.count_nonzero(mask[y:y+h,x:x+w])
color_analysis(image, mask, '/Users/aravera/Documents/PROJECTS/DSB_Salamin/Lucy_s35/res/', 'a', 5)"""


"""
This module contains four napari widgets declared in
different ways:

- a pure Python function flagged with `autogenerate: true`
    in the plugin manifest. Type annotations are used by
    magicgui to generate widgets for each parameter. Best
    suited for simple processing tasks - usually taking
    in and/or returning a layer.
- a `magic_factory` decorated function. The `magic_factory`
    decorator allows us to customize aspects of the resulting
    GUI, including the widgets associated with each parameter.
    Best used when you have a very simple processing task,
    but want some control over the autogenerated widgets. If you
    find yourself needing to define lots of nested functions to achieve
    your functionality, maybe look at the `Container` widget!
- a `magicgui.widgets.Container` subclass. This provides lots
    of flexibility and customization options while still supporting
    `magicgui` widgets and convenience methods for creating widgets
    from type annotations. If you want to customize your widgets and
    connect callbacks, this is the best widget option for you.
- a `QWidget` subclass. This provides maximal flexibility but requires
    full specification of widget layouts, callbacks, events, etc.

References:
- Widget specification: https://napari.org/stable/plugins/guides.html?#widgets
- magicgui docs: https://pyapp-kit.github.io/magicgui/

Replace code below according to your needs.
"""
from typing import TYPE_CHECKING

import os
import cv2
import numpy as np
from magicgui.widgets import create_widget, PushButton, Container, ToolBar, SpinBox, LineEdit, ComboBox
from qtpy.QtWidgets import QFileDialog

from .functions import color_analysis, bw_correction, rgb_correction, cmy_correction, set_cm_scale, lock_controls

import warnings
warnings.filterwarnings('ignore')


if TYPE_CHECKING:
    import napari

from functools import partial


class FishAnalysis(Container):
    def __init__(self, viewer: "napari.viewer.Viewer"):
        super().__init__()
        self._viewer = viewer
        self.rect_layer = None
        self.line_layer = None
        self.fishmask_layer = None
        self.rotation = 0
        self.cmscale = None
        self.default = None
        self.previous_corrected_layer = None


        self.tb = ToolBar()
        
        ################################################################
        self.c1 = Container(name="Rotation")
        # Select file
        self._image_layer_rot = create_widget(
            label="Image", annotation="napari.layers.Image"
        )
        # Select rotation angle
        self.rotationangle = SpinBox(label="Rotation angle", value=0, min=-180, max=180)
        # Button rotate image
        self.btn_rotate = PushButton(value=True, text='Rotate')
        self.btn_rotate.clicked.connect(self._on_click_rotate)

        self.c1.extend(
            [
                self.tb,
                self._image_layer_rot,
                self.rotationangle,
                self.btn_rotate,
            ]
        )

        ################################################################
        self.c2 = Container(name ="Color correction")
        # Select file
        self._image_layer_col = create_widget(
            label="Image", annotation="napari.layers.Image"
        )
        # Button shape layer
        self.btn_rectangle = PushButton(value=True, text='Select the area')
        self.btn_rectangle.clicked.connect(lambda: self._on_click_createlayer('color_rectangle'))
        # Color combobox
        self.colorbox = ComboBox(choices=['white', 'black', 'red', 'green', 'blue', 'cyan', 'magenta', 'yellow'])
        # Button correct color
        self.btn_correct = PushButton(value=True, text='Correct color')
        self.btn_correct.clicked.connect(self._on_click_colorcorrection)
        # Button correct color
        self.btn_median = PushButton(value=True, text='Apply median filter')
        self.btn_median.clicked.connect(self._on_click_medianfilter)

        self.c2.extend(
            [
                self.tb,
                self._image_layer_col,
                self.btn_rectangle,
                self.colorbox,
                self.btn_correct,
                self.btn_median,
            ]
        )

        ################################################################
        self.c3 = Container(name="Set scale")
        # Button shape layer
        self.btn_line = PushButton(value=True, text='Select 1 cm')
        self.btn_line.clicked.connect(lambda: self._on_click_createlayer('line'))
        # Button correct color
        self.btn_setscale = PushButton(value=True, text='Set scale')
        self.btn_setscale.clicked.connect(self._on_click_cmscale)

        self.c3.extend(
            [
                self.tb,
                self.btn_line,
                self.btn_setscale,
            ]
        )

        ################################################################
        self.c4 = Container(name='Segmentation')
        # Select file
        self._image_layer_seg = create_widget(
            label="Image", annotation="napari.layers.Image"
        )
        # Button label layer
        self.btn_rectangle1 = PushButton(value=True, text='Select fish area')
        self.btn_rectangle1.clicked.connect(self._on_click_createmask)

        self.c4.extend(
            [
                self.tb,
                self._image_layer_seg,
                self.btn_rectangle1
            ]
        )

        ################################################################
        self.c5 = Container(name='Analysis')
        # Define output directory
        self.saving_dir = LineEdit(label="Output folder")
        self.browse_btn = PushButton(value=True, text='Browse')
        self.browse_btn.clicked.connect(self._add_folder)
        # Button analysis
        self.btn_analyse = PushButton(value=True, text='Analyse the fish')
        self.btn_analyse.clicked.connect(self._on_click_analyse)

        self.c5.extend(
            [
                self.saving_dir,
                self.browse_btn,
                self.btn_analyse,
            ]
        )
        ################################################################
        self.extend(
            [
                #self.c1,
                self.c2,
                self.c3,
                self.c4,
                self.c5,
            ]
        )

    
    def _add_folder(self):
        # Add the selected mask as input
        self.saving_dir.value = QFileDialog.getExistingDirectory()


    def _on_click_rotate(self):
        """image_layer = self._image_layer_rot.value
        if image_layer is None:
            print("Select image layer before rotating")
            return"""

        names = [x.name for x in self._viewer.layers]

        for image_name in names:
            img = np.asarray(self._viewer.layers[image_name].data)
            self._viewer.layers.remove(image_name)
            self._viewer.add_image(img, name=image_name, rgb=True, rotate=self.rotationangle.value)
            self.rotation = self.rotationangle.value


    def _on_click_createlayer(self, t):
        if t == 'color_rectangle':
            self.rect_layer = self._viewer.add_shapes(name='Rectangle', shape_type='rectangle', edge_width=5, edge_color='red')
            widget_list_toblock = [
                'ellipse_button',
                'line_button',
                'path_button',
                'polygon_button',
                'polygon_lasso_button',
                'vertex_remove_button',
                'vertex_insert_button',
                'move_back_button',
                'move_front_button'
            ]
            lock_controls(self.rect_layer, self._viewer, widget_list_toblock)
        else:
            self.line_layer = self._viewer.add_shapes(name='Line', shape_type='line', edge_width=5, edge_color='red')
            widget_list_toblock = [
                'ellipse_button',
                'rectangle_button',
                'path_button',
                'polygon_button',
                'polygon_lasso_button',
                'vertex_remove_button',
                'vertex_insert_button',
                'move_back_button',
                'move_front_button'
            ]
            lock_controls(self.line_layer, self._viewer, widget_list_toblock)


    def _on_click_colorcorrection(self):
        # Read image and its name (cleaned)
        image_layer = self._image_layer_col.value
        if image_layer is None:
            print("Select image layer before correcting color")
            return
        image = np.asarray(image_layer.data)
        image_name = image_layer.name.replace('Current_corrected_', '').replace('Previous_corrected_', '')
        # Read the color to process
        color = self.colorbox.value
        # Process
        if color == 'white':
            new_image = bw_correction(image, self.rect_layer, 'white')
        elif color == 'black':
            new_image = bw_correction(image, self.rect_layer, 'black')
        elif color in ['red', 'green', 'blue']:
            new_image = rgb_correction(image, self.rect_layer, color)
        else:
            new_image = cmy_correction(image, self.rect_layer, color)

        # Add the new layer
        previous_name = 'Previous_corrected_'+image_name
        current_name = 'Current_corrected_'+image_name
        layers_names = self._viewer.layers
        
        ## If there are no images OR
        ## If there is only the previous (it means they deleted the current)
        ## just insert the current image
        if (previous_name not in layers_names and current_name not in layers_names) or (previous_name in layers_names and current_name not in layers_names):
            self._viewer.add_image(new_image, name=current_name, rgb=True, rotate=self.rotation)

        ## If there are is a currect images, insert as current and move the previous
        elif previous_name not in layers_names and current_name in layers_names:
            # Remove current and save it as previous
            self.previous_corrected_layer = self._viewer.layers[current_name]
            self._viewer.add_image(self.previous_corrected_layer.data, name=previous_name, rgb=True, rotate=self.rotation)
            self._viewer.layers.remove(current_name)
            # Save new current
            self._viewer.add_image(new_image, name=current_name, rgb=True, rotate=self.rotation)

        ## If there are both images, insert as current the new one, move the previous, delete the other
        elif previous_name in layers_names and current_name in layers_names:
            # Delete the previous
            self._viewer.layers.remove(previous_name)
            # Save the new previous
            self.previous_corrected_layer = self._viewer.layers[current_name]
            self._viewer.add_image(self.previous_corrected_layer.data, name=previous_name, rgb=True, rotate=self.rotation)
            # Detete the current (now previous)
            self._viewer.layers.remove(current_name)
            # Save new current
            self._viewer.add_image(new_image, name=current_name, rgb=True, rotate=self.rotation)
            
        # Delete the rectangle layer
        if self.rect_layer is not None:
            self._viewer.layers.remove('Rectangle')
            self.rect_layer = None
        

    def _on_click_medianfilter(self):
        image_layer = self._image_layer_col.value
        if image_layer is None:
            print("Select image layer before correcting color")
            return
        image = np.asarray(image_layer.data)
        name = image_layer.name
        self._viewer.layers.remove(name)
        median = cv2.medianBlur(image, 5)
        self._viewer.add_image(median, name=name, rgb=True, rotate=self.rotation)


    def _on_click_cmscale(self):
        if self.line_layer.data is None:
            print("Create a line before setting scale")
            return
        self.cmscale = set_cm_scale(self.line_layer.data)
        self._viewer.layers.remove('Line')


    def _on_click_createmask(self):
        image_layer = self._image_layer_seg.value
        if image_layer is None:
            print("Select image layer before segmenting")
            return
        image = np.asarray(image_layer.data)
        self.fishmask_layer = self._viewer.add_labels(np.zeros((image.shape[0], image.shape[1]), dtype=np.uint8), name='Fish mask')


    def _on_click_analyse(self):
        image_layer = self._image_layer_seg.value
        if image_layer is None:
            print("Select image layer before segmenting")
            return
        mask = self.fishmask_layer.data
        image = np.asarray(image_layer.data)
        image_name = image_layer.name.replace('Current_corrected_', '').replace('Previous_corrected_', '')
        if self.saving_dir.value != '':
            output_path = self.saving_dir.value
        else:
            output_path = os.path.dirname(image_layer.source.path)
        color_analysis(image, mask, output_path, image_name ,self.cmscale)
        